@@ -1,5 +1,3 @@
-from __future__ import division, absolute_import, print_function
-
 # doctest
 r''' Test the .npy file format.
 
@@ -278,8 +276,6 @@ Test the header writing.
 '''
 import sys
 import os
-import shutil
-import tempfile
 import warnings
 import pytest
 from io import BytesIO
@@ -287,28 +283,12 @@ from io import BytesIO
 import numpy as np
 from numpy.testing import (
     assert_, assert_array_equal, assert_raises, assert_raises_regex,
-    assert_warns
+    assert_warns, IS_PYPY,
     )
+from numpy.testing._private.utils import requires_memory
 from numpy.lib import format
 
 
-tempdir = None
-
-# Module-level setup.
-
-
-def setup_module():
-    global tempdir
-    tempdir = tempfile.mkdtemp()
-
-
-def teardown_module():
-    global tempdir
-    if tempdir is not None and os.path.isdir(tempdir):
-        shutil.rmtree(tempdir)
-        tempdir = None
-
-
 # Generate some basic arrays to test with.
 scalars = [
     np.uint8,
@@ -421,7 +401,7 @@ class BytesIOSRandomSize(BytesIO):
     def read(self, size=None):
         import random
         size = random.randint(1, size)
-        return super(BytesIOSRandomSize, self).read(size)
+        return super().read(size)
 
 
 def roundtrip(arr):
@@ -479,49 +459,42 @@ def test_long_str():
     assert_array_equal(long_str_arr, long_str_arr2)
 
 
-@pytest.mark.slow
-def test_memmap_roundtrip():
-    # Fixme: used to crash on windows
-    if not (sys.platform == 'win32' or sys.platform == 'cygwin'):
-        for arr in basic_arrays + record_arrays:
-            if arr.dtype.hasobject:
-                # Skip these since they can't be mmap'ed.
-                continue
-            # Write it out normally and through mmap.
-            nfn = os.path.join(tempdir, 'normal.npy')
-            mfn = os.path.join(tempdir, 'memmap.npy')
-            fp = open(nfn, 'wb')
-            try:
-                format.write_array(fp, arr)
-            finally:
-                fp.close()
+def test_memmap_roundtrip(tmpdir):
+    for i, arr in enumerate(basic_arrays + record_arrays):
+        if arr.dtype.hasobject:
+            # Skip these since they can't be mmap'ed.
+            continue
+        # Write it out normally and through mmap.
+        nfn = os.path.join(tmpdir, f'normal{i}.npy')
+        mfn = os.path.join(tmpdir, f'memmap{i}.npy')
+        with open(nfn, 'wb') as fp:
+            format.write_array(fp, arr)
 
-            fortran_order = (
-                arr.flags.f_contiguous and not arr.flags.c_contiguous)
-            ma = format.open_memmap(mfn, mode='w+', dtype=arr.dtype,
-                                    shape=arr.shape, fortran_order=fortran_order)
-            ma[...] = arr
-            del ma
+        fortran_order = (
+            arr.flags.f_contiguous and not arr.flags.c_contiguous)
+        ma = format.open_memmap(mfn, mode='w+', dtype=arr.dtype,
+                                shape=arr.shape, fortran_order=fortran_order)
+        ma[...] = arr
+        ma.flush()
 
-            # Check that both of these files' contents are the same.
-            fp = open(nfn, 'rb')
+        # Check that both of these files' contents are the same.
+        with open(nfn, 'rb') as fp:
             normal_bytes = fp.read()
-            fp.close()
-            fp = open(mfn, 'rb')
+        with open(mfn, 'rb') as fp:
             memmap_bytes = fp.read()
-            fp.close()
-            assert_equal_(normal_bytes, memmap_bytes)
+        assert_equal_(normal_bytes, memmap_bytes)
 
-            # Check that reading the file using memmap works.
-            ma = format.open_memmap(nfn, mode='r')
-            del ma
+        # Check that reading the file using memmap works.
+        ma = format.open_memmap(nfn, mode='r')
+        ma.flush()
 
 
-def test_compressed_roundtrip():
+def test_compressed_roundtrip(tmpdir):
     arr = np.random.rand(200, 200)
-    npz_file = os.path.join(tempdir, 'compressed.npz')
+    npz_file = os.path.join(tmpdir, 'compressed.npz')
     np.savez_compressed(npz_file, arr=arr)
-    arr1 = np.load(npz_file)['arr']
+    with np.load(npz_file) as npz:
+        arr1 = npz['arr']
     assert_array_equal(arr, arr1)
 
 
@@ -537,23 +510,23 @@ dt4 = np.dtype({'names': ['a', '', 'b'], 'formats': ['i4']*3})
 # titles
 dt5 = np.dtype({'names': ['a', 'b'], 'formats': ['i4', 'i4'],
                 'offsets': [1, 6], 'titles': ['aa', 'bb']})
+# empty
+dt6 = np.dtype({'names': [], 'formats': [], 'itemsize': 8})
 
-@pytest.mark.parametrize("dt", [dt1, dt2, dt3, dt4, dt5])
-def test_load_padded_dtype(dt):
+@pytest.mark.parametrize("dt", [dt1, dt2, dt3, dt4, dt5, dt6])
+def test_load_padded_dtype(tmpdir, dt):
     arr = np.zeros(3, dt)
     for i in range(3):
         arr[i] = i + 5
-    npz_file = os.path.join(tempdir, 'aligned.npz')
+    npz_file = os.path.join(tmpdir, 'aligned.npz')
     np.savez(npz_file, arr=arr)
-    arr1 = np.load(npz_file)['arr']
+    with np.load(npz_file) as npz:
+        arr1 = npz['arr']
     assert_array_equal(arr, arr1)
 
 
 def test_python2_python3_interoperability():
-    if sys.version_info[0] >= 3:
-        fname = 'win64python2.npy'
-    else:
-        fname = 'python3.npy'
+    fname = 'win64python2.npy'
     path = os.path.join(os.path.dirname(__file__), 'data', fname)
     data = np.load(path)
     assert_array_equal(data, np.ones(2))
@@ -563,13 +536,7 @@ def test_pickle_python2_python3():
     # Python 2 and Python 3 and vice versa
     data_dir = os.path.join(os.path.dirname(__file__), 'data')
 
-    if sys.version_info[0] >= 3:
-        xrange = range
-    else:
-        import __builtin__
-        xrange = __builtin__.xrange
-
-    expected = np.array([None, xrange, u'\u512a\u826f',
+    expected = np.array([None, range, u'\u512a\u826f',
                          b'\xe4\xb8\x8d\xe8\x89\xaf'],
                         dtype=object)
 
@@ -585,37 +552,33 @@ def test_pickle_python2_python3():
             else:
                 data = data_f
 
-            if sys.version_info[0] >= 3:
-                if encoding == 'latin1' and fname.startswith('py2'):
-                    assert_(isinstance(data[3], str))
-                    assert_array_equal(data[:-1], expected[:-1])
-                    # mojibake occurs
-                    assert_array_equal(data[-1].encode(encoding), expected[-1])
-                else:
-                    assert_(isinstance(data[3], bytes))
-                    assert_array_equal(data, expected)
+            if encoding == 'latin1' and fname.startswith('py2'):
+                assert_(isinstance(data[3], str))
+                assert_array_equal(data[:-1], expected[:-1])
+                # mojibake occurs
+                assert_array_equal(data[-1].encode(encoding), expected[-1])
             else:
+                assert_(isinstance(data[3], bytes))
                 assert_array_equal(data, expected)
 
-        if sys.version_info[0] >= 3:
-            if fname.startswith('py2'):
-                if fname.endswith('.npz'):
-                    data = np.load(path, allow_pickle=True)
-                    assert_raises(UnicodeError, data.__getitem__, 'x')
-                    data.close()
-                    data = np.load(path, allow_pickle=True, fix_imports=False,
-                                   encoding='latin1')
-                    assert_raises(ImportError, data.__getitem__, 'x')
-                    data.close()
-                else:
-                    assert_raises(UnicodeError, np.load, path,
-                                  allow_pickle=True)
-                    assert_raises(ImportError, np.load, path,
-                                  allow_pickle=True, fix_imports=False,
-                                  encoding='latin1')
+        if fname.startswith('py2'):
+            if fname.endswith('.npz'):
+                data = np.load(path, allow_pickle=True)
+                assert_raises(UnicodeError, data.__getitem__, 'x')
+                data.close()
+                data = np.load(path, allow_pickle=True, fix_imports=False,
+                               encoding='latin1')
+                assert_raises(ImportError, data.__getitem__, 'x')
+                data.close()
+            else:
+                assert_raises(UnicodeError, np.load, path,
+                              allow_pickle=True)
+                assert_raises(ImportError, np.load, path,
+                              allow_pickle=True, fix_imports=False,
+                              encoding='latin1')
 
 
-def test_pickle_disallow():
+def test_pickle_disallow(tmpdir):
     data_dir = os.path.join(os.path.dirname(__file__), 'data')
 
     path = os.path.join(data_dir, 'py2-objarr.npy')
@@ -623,10 +586,10 @@ def test_pickle_disallow():
                   allow_pickle=False, encoding='latin1')
 
     path = os.path.join(data_dir, 'py2-objarr.npz')
-    f = np.load(path, allow_pickle=False, encoding='latin1')
-    assert_raises(ValueError, f.__getitem__, 'x')
+    with np.load(path, allow_pickle=False, encoding='latin1') as f:
+        assert_raises(ValueError, f.__getitem__, 'x')
 
-    path = os.path.join(tempdir, 'pickle-disabled.npy')
+    path = os.path.join(tmpdir, 'pickle-disabled.npy')
     assert_raises(ValueError, np.save, path, np.array([None], dtype=object),
                   allow_pickle=False)
 
@@ -711,31 +674,33 @@ def test_version_2_0():
     assert_raises(ValueError, format.write_array, f, d, (1, 0))
 
 
-@pytest.mark.slow
-def test_version_2_0_memmap():
+def test_version_2_0_memmap(tmpdir):
     # requires more than 2 byte for header
     dt = [(("%d" % i) * 100, float) for i in range(500)]
     d = np.ones(1000, dtype=dt)
-    tf = tempfile.mktemp('', 'mmap', dir=tempdir)
+    tf1 = os.path.join(tmpdir, f'version2_01.npy')
+    tf2 = os.path.join(tmpdir, f'version2_02.npy')
 
     # 1.0 requested but data cannot be saved this way
-    assert_raises(ValueError, format.open_memmap, tf, mode='w+', dtype=d.dtype,
+    assert_raises(ValueError, format.open_memmap, tf1, mode='w+', dtype=d.dtype,
                             shape=d.shape, version=(1, 0))
 
-    ma = format.open_memmap(tf, mode='w+', dtype=d.dtype,
+    ma = format.open_memmap(tf1, mode='w+', dtype=d.dtype,
                             shape=d.shape, version=(2, 0))
     ma[...] = d
-    del ma
+    ma.flush()
+    ma = format.open_memmap(tf1, mode='r')
+    assert_array_equal(ma, d)
 
     with warnings.catch_warnings(record=True) as w:
         warnings.filterwarnings('always', '', UserWarning)
-        ma = format.open_memmap(tf, mode='w+', dtype=d.dtype,
+        ma = format.open_memmap(tf2, mode='w+', dtype=d.dtype,
                                 shape=d.shape, version=None)
         assert_(w[0].category is UserWarning)
         ma[...] = d
-        del ma
+        ma.flush()
 
-    ma = format.open_memmap(tf, mode='r')
+    ma = format.open_memmap(tf2, mode='r')
     assert_array_equal(ma, d)
 
 
@@ -887,11 +852,11 @@ def test_bad_header():
     assert_raises(ValueError, format.read_array_header_1_0, s)
 
 
-def test_large_file_support():
+def test_large_file_support(tmpdir):
     if (sys.platform == 'win32' or sys.platform == 'cygwin'):
         pytest.skip("Unknown if Windows has sparse filesystems")
     # try creating a large sparse file
-    tf_name = os.path.join(tempdir, 'sparse_file')
+    tf_name = os.path.join(tmpdir, 'sparse_file')
     try:
         # seek past end would work too, but linux truncate somewhat
         # increases the chances that we have a sparse filesystem and can
@@ -915,33 +880,38 @@ def test_large_file_support():
 @pytest.mark.skipif(np.dtype(np.intp).itemsize < 8,
                     reason="test requires 64-bit system")
 @pytest.mark.slow
-def test_large_archive():
+@requires_memory(free_bytes=2 * 2**30)
+def test_large_archive(tmpdir):
     # Regression test for product of saving arrays with dimensions of array
     # having a product that doesn't fit in int32.  See gh-7598 for details.
+    shape = (2**30, 2)
     try:
-        a = np.empty((2**30, 2), dtype=np.uint8)
+        a = np.empty(shape, dtype=np.uint8)
     except MemoryError:
         pytest.skip("Could not create large file")
 
-    fname = os.path.join(tempdir, "large_archive")
+    fname = os.path.join(tmpdir, "large_archive")
 
     with open(fname, "wb") as f:
         np.savez(f, arr=a)
 
+    del a
+
     with open(fname, "rb") as f:
         new_a = np.load(f)["arr"]
 
-    assert_(a.shape == new_a.shape)
+    assert new_a.shape == shape
 
 
-def test_empty_npz():
+def test_empty_npz(tmpdir):
     # Test for gh-9989
-    fname = os.path.join(tempdir, "nothing.npz")
+    fname = os.path.join(tmpdir, "nothing.npz")
     np.savez(fname)
-    np.load(fname)
+    with np.load(fname) as nps:
+        pass
 
 
-def test_unicode_field_names():
+def test_unicode_field_names(tmpdir):
     # gh-7391
     arr = np.array([
         (1, 3),
@@ -952,7 +922,7 @@ def test_unicode_field_names():
         ('int', int),
         (u'\N{CJK UNIFIED IDEOGRAPH-6574}\N{CJK UNIFIED IDEOGRAPH-5F62}', int)
     ])
-    fname = os.path.join(tempdir, "unicode.npy")
+    fname = os.path.join(tmpdir, "unicode.npy")
     with open(fname, 'wb') as f:
         format.write_array(f, arr, version=(3, 0))
     with open(fname, 'rb') as f:
@@ -975,6 +945,8 @@ def test_unicode_field_names():
         float, np.dtype({'names': ['c'], 'formats': [np.dtype(int, metadata={})]})
     ]}), False)
     ])
+@pytest.mark.skipif(IS_PYPY and sys.implementation.version <= (7, 3, 8),
+        reason="PyPy bug in error formatting")
 def test_metadata_dtype(dt, fail):
     # gh-14142
     arr = np.ones(10, dtype=dt)
